from collections import deque, defaultdict


def find_protected_warehouses(n, m, k, roads, guards):
    graph = defaultdict(list)
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)
    queue = deque()
    distances = [-1] * (n + 1)
    for p, h in guards:
        queue.append((p, h))
        distances[p] = max(distances[p], h)
    while queue:
        current, stamina = queue.popleft()
        if stamina == 0:
            continue
        for neighbor in graph[current]:
            if distances[neighbor] < stamina - 1:
                distances[neighbor] = stamina - 1
                queue.append((neighbor, stamina - 1))
    protected = [i for i in range(1, n + 1) if distances[i] >= 0]
    return protected


def main():
    n, m, k = map(int, input().split())

    roads = []
    for _ in range(m):
        a, b = map(int, input().split())
        roads.append((a, b))

    guards = []
    for _ in range(k):
        p, h = map(int, input().split())
        guards.append((p, h))

    protected = find_protected_warehouses(n, m, k, roads, guards)
    print(len(protected))
    if protected:
        print(" ".join(map(str, sorted(protected))))
    else:
        print()


if __name__ == "__main__":
    main()
